#include <Adafruit_NeoPixel.h>

#define PURE_BINARY

#define LED_PIN            4
#define PIN_UP             0 
#define PIN_DOWN           1 
#define PIN_ON_OFF         2 
#define LED_COUNT          17
#define MILLIS_PER_SECOND  1000
#define MILLIS_PER_DAY     86400000

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

unsigned long systemTime; // Keep track of the millis() counter (in milliseconds).
long displayedTime;       // Displayed Time (in milliseconds).
long numLoops;
bool ledsAreOn;
long secondsOld;

// Set up the LEDs and the "timer".
void setup(){
  ledsAreOn = true;
  displayedTime = 0;
  secondsOld = 0;
  systemTime = millis();
  pixels.begin();
  pinMode(PIN_ON_OFF, INPUT_PULLUP);
  pinMode(PIN_DOWN, INPUT_PULLUP);
  pinMode(PIN_UP, INPUT_PULLUP);
  numLoops = 0;
}

void loop(){
  char textBuffer[128];
  long hours, minutes, seconds;
  int i;
  bool hasUpdate = false;
  long tuningDirection;

  // ****************** Calculate the current time.
  if(millis() - systemTime > MILLIS_PER_SECOND){
    systemTime += MILLIS_PER_SECOND;
    displayedTime = (displayedTime + MILLIS_PER_SECOND) % MILLIS_PER_DAY;
    hasUpdate = true;
    if(!digitalRead(PIN_ON_OFF)){ // Switch all LEDs on/off.
      ledsAreOn = !ledsAreOn;
    }
  }
  if(!ledsAreOn){
    for(i = 0; i < 17; i++){
      pixels.setPixelColor(i, 0x000000); 
    }
    pixels.show(); // This sends the updated pixel color to the hardware.
    delay(100);
    return;
  }
  tuningDirection = !digitalRead(PIN_UP) - !digitalRead(PIN_DOWN);
  if(tuningDirection != 0){
    displayedTime = (displayedTime + tuningDirection * 10 * numLoops + MILLIS_PER_DAY) % MILLIS_PER_DAY;
    numLoops++;
    delay(5);
    hasUpdate = true;
  }
  else{     
    numLoops = 0;
  }
  seconds = displayedTime / MILLIS_PER_SECOND;
  hours = seconds / 3600;
  seconds = seconds % 3600;
  minutes = seconds / 60;
  seconds = seconds % 60;
  if(!hasUpdate){
    return;
  }
  // ****************** Let's switch the LEDs on/off!
  // Hour LEDs.
  for(i = 0; i < 5; i++){
    pixels.setPixelColor(i, (0x0000FF + i * 0x101000) * ((hours >> (4 - i)) & 0x01)); 
  }
#ifdef PURE_BINARY // Display minutes and secons in real binary coding.
  // Minutes LEDs.
  for(i = 0; i < 6; i++){
    pixels.setPixelColor(i+5, (0x00FF00 + i * 0x010010) * ((minutes >> (5 - i)) & 0x01)); 
  }
  // Seconds LEDs.
  for(i = 0; i < 6; i++){
    pixels.setPixelColor(i+11, (0xFF0000 + i * 0x000404) * ((seconds >> (5 - i)) & 0x01)); 
  }
#else // Display minutes and seconds using the first LED as "30" the second LED as "15" and the remaining 4 as "0...14".
  // Minutes LEDs.
  // Set 30 minutes LED.
  pixels.setPixelColor(5, 0x00FF00*(long)(minutes >= 30));
  minutes %= 30;
  // Set 15 minutes LED.
  pixels.setPixelColor(6, 0x10FF10*(long)(minutes >= 15));
  minutes %= 15;
  // Set 0...14 minutes LEDs.
  for(i = 2; i < 6; i++){
    pixels.setPixelColor(i+5, (0x00FF00 + i * 0x100010) * ((minutes >> (5 - i)) & 0x01)); 
  }
  // Seconds LEDs.
  // Set 30 seconds LED.
  pixels.setPixelColor(11, 0xFF0000*(long)(seconds >= 30));
  seconds %= 30;
  // Set 15 minutes LED.
  pixels.setPixelColor(12, 0xFF1010*(long)(seconds >= 15));
  seconds %= 15;
  // Set 0...14 seconds LEDs.
  for(i = 2; i < 6; i++){
    pixels.setPixelColor(i+11, (0xFF0000 + i * 0x001010) * ((seconds >> (5 - i)) & 0x01)); 
  }
#endif
  pixels.show(); // This sends the updated pixel color to the hardware.
}